@@ -1,6 +1,6 @@
 use super::PositiveDefiniteKernel;
-use crate::Value;
 use crate::{KernelAdd, KernelError, KernelMul};
+use crate::{ParamsDifferentiableKernel, Value, ValueDifferentiableKernel};
 use std::fmt::Debug;
 use std::{ops::Add, ops::Mul};
 
@@ -50,6 +50,30 @@ where
     }
 }
 
+impl ValueDifferentiableKernel<Vec<f64>> for Constant {
+    fn ln_diff_value(
+        &self,
+        _params: &[f64],
+        x: &Vec<f64>,
+        _xprime: &Vec<f64>,
+    ) -> Result<Vec<f64>, KernelError> {
+        let diff = vec![0.0; x.len()];
+        Ok(diff)
+    }
+}
+
+impl ParamsDifferentiableKernel<Vec<f64>> for Constant {
+    fn ln_diff_params(
+        &self,
+        _params: &[f64],
+        _x: &Vec<f64>,
+        _xprime: &Vec<f64>,
+    ) -> Result<Vec<f64>, KernelError> {
+        let diff = vec![1.0];
+        Ok(diff)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::*;
