@@ -1,5 +1,8 @@
 use super::PositiveDefiniteKernel;
-use crate::{KernelAdd, KernelError, KernelMul};
+use crate::{
+    KernelAdd, KernelError, KernelMul, ParamsDifferentiableKernel, ValueDifferentiableKernel,
+};
+use opensrdk_linear_algebra::Vector;
 use rayon::prelude::*;
 use std::{ops::Add, ops::Mul};
 
@@ -42,6 +45,31 @@ impl PositiveDefiniteKernel<Vec<f64>> for Exponential {
     }
 }
 
+impl ValueDifferentiableKernel<Vec<f64>> for Exponential {
+    fn ln_diff_value(
+        &self,
+        params: &[f64],
+        x: &Vec<f64>,
+        xprime: &Vec<f64>,
+    ) -> Result<Vec<f64>, KernelError> {
+        let diff = (-2.0 / params[0] * (x.clone().col_mat() - xprime.clone().col_mat())).vec();
+        Ok(diff)
+    }
+}
+
+impl ParamsDifferentiableKernel<Vec<f64>> for Exponential {
+    fn ln_diff_params(
+        &self,
+        params: &[f64],
+        x: &Vec<f64>,
+        xprime: &Vec<f64>,
+    ) -> Result<Vec<f64>, KernelError> {
+        let diff1 = 2.0 * params[0].powi(-2) * &self.norm(params, x, xprime).unwrap();
+        let diff = vec![diff1];
+        Ok(diff)
+    }
+}
+
 impl<R> Add<R> for Exponential
 where
     R: PositiveDefiniteKernel<Vec<f64>>,
